@@ -10,7 +10,7 @@ import os
 import numpy as np
 import pandas as pd
 import scipy
-from scipy.sparse import issparse
+from scipy.sparse import issparse, csr
 
 import torch
 from torch.utils.data import Dataset
@@ -86,6 +86,9 @@ def load_file(path):
             adata = AnnData(df.values, dict(obs_names=df.index.values), dict(var_names=df.columns.values))
         elif path.endswith('.h5ad'):
             adata = sc.read_h5ad(path)
+    elif path.endswith(tuple(['.h5mu/rna', '.h5mu/atac'])):
+        import muon as mu
+        adata = mu.read(path)
     else:
         raise ValueError("File {} not exists".format(path))
         
@@ -151,16 +154,22 @@ def concat_data(
         return load_files(data_list[0])
     elif isinstance(data_list, str):
         return load_files(data_list)
+    elif isinstance(data_list, AnnData):
+        return data_list
+
     adata_list = []
     for root in data_list:
-        adata = load_files(root)
+        if isinstance(root, AnnData):
+            adata = root
+        else:
+            adata = load_files(root)
         adata_list.append(adata)
         
     if batch_categories is None:
         batch_categories = list(map(str, range(len(adata_list))))
     else:
         assert len(adata_list) == len(batch_categories)
-    [print(b, adata.shape) for adata,b in zip(adata_list, batch_categories)]
+    # [print(b, adata.shape) for adata,b in zip(adata_list, batch_categories)]
     concat = AnnData.concatenate(*adata_list, join=join, batch_key=batch_key,
                                 batch_categories=batch_categories, index_unique=index_unique)  
     if save:
@@ -203,9 +212,11 @@ def preprocessing_rna(
     """
     if min_features is None: min_features = 600
     if n_top_features is None: n_top_features = 2000
+    if target_sum is None: target_sum = 10000
     
     if log: log.info('Preprocessing')
-    if not issparse(adata.X):
+    # if not issparse(adata.X):
+    if type(adata.X) != csr.csr_matrix:
         adata.X = scipy.sparse.csr_matrix(adata.X)
     
     adata = adata[:, [gene for gene in adata.var_names 
@@ -241,7 +252,7 @@ def preprocessing_atac(
         min_features: int = 100, 
         min_cells: int = 3, 
         target_sum=None, 
-        n_top_features = 30000, # or gene list
+        n_top_features = 100000, # or gene list
         chunk_size: int = CHUNK_SIZE,
         log=None
     ):
@@ -272,10 +283,12 @@ def preprocessing_atac(
     import episcanpy as epi
     
     if min_features is None: min_features = 100
-    if n_top_features is None: n_top_features = 30000
+    if n_top_features is None: n_top_features = 100000
+    if target_sum is None: target_sum = 10000
     
     if log: log.info('Preprocessing')
-    if not issparse(adata.X):
+    # if not issparse(adata.X):
+    if type(adata.X) != csr.csr_matrix:
         adata.X = scipy.sparse.csr_matrix(adata.X)
     
     adata.X[adata.X>1] = 1
@@ -289,13 +302,15 @@ def preprocessing_atac(
 #     adata.raw = adata
     if log: log.info('Finding variable features')
     if type(n_top_features) == int and n_top_features>0:
-        epi.pp.select_var_feature(adata, nb_features=n_top_features, show=False, copy=False)
+        sc.pp.highly_variable_genes(adata, n_top_genes=n_top_features, batch_key='batch', inplace=False, subset=True)
+        # epi.pp.select_var_feature(adata, nb_features=n_top_features, show=False, copy=False)
     elif type(n_top_features) != int:
         adata = reindex(adata, n_top_features)
 
     
-    if log: log.info('Normalizing total per cell')
-    sc.pp.normalize_total(adata, target_sum=target_sum)
+    # if log: log.info('Normalizing total per cell')
+    # if target_sum != -1:
+        # sc.pp.normalize_total(adata, target_sum=target_sum)
 
         
     if log: log.info('Batch specific maxabs scaling')
@@ -309,8 +324,8 @@ def preprocessing(
         profile: str='RNA',
         min_features: int = 600, 
         min_cells: int = 3, 
-        target_sum: int = 10000, 
-        n_top_features = 2000, # or gene list
+        target_sum: int = None, 
+        n_top_features = None, # or gene list
         chunk_size: int = CHUNK_SIZE,
         log=None
     ):
@@ -346,6 +361,7 @@ def preprocessing(
                    adata, 
                    min_features=min_features, 
                    min_cells=min_cells, 
+                   target_sum=target_sum,
                    n_top_features=n_top_features, 
                    chunk_size=chunk_size, 
                    log=log
@@ -355,11 +371,13 @@ def preprocessing(
                    adata, 
                    min_features=min_features, 
                    min_cells=min_cells, 
+                   target_sum=target_sum,
                    n_top_features=n_top_features, 
                    chunk_size=chunk_size, 
                    log=log
                )
-    
+    else:
+        raise ValueError("Not support profile: `{}` yet".format(profile))
 
 def batch_scale(adata, chunk_size=CHUNK_SIZE):
     """
@@ -404,10 +422,13 @@ def reindex(adata, genes, chunk_size=CHUNK_SIZE):
     """
     idx = [i for i, g in enumerate(genes) if g in adata.var_names]
     print('There are {} gene in selected genes'.format(len(idx)))
-    new_X = scipy.sparse.csr_matrix((adata.shape[0], len(genes)))
-    for i in range(new_X.shape[0]//chunk_size+1):
-        new_X[i*chunk_size:(i+1)*chunk_size, idx] = adata[i*chunk_size:(i+1)*chunk_size, genes[idx]].X
-    adata = AnnData(new_X, obs=adata.obs, var={'var_names':genes}) 
+    if len(idx) == len(genes):
+        adata = adata[:, genes]
+    else:
+        new_X = scipy.sparse.csr_matrix((adata.shape[0], len(genes)))
+        for i in range(new_X.shape[0]//chunk_size+1):
+            new_X[i*chunk_size:(i+1)*chunk_size, idx] = adata[i*chunk_size:(i+1)*chunk_size, genes[idx]].X
+        adata = AnnData(new_X, obs=adata.obs, var={'var_names':genes}) 
     return adata
 
 
@@ -492,9 +513,11 @@ def load_data(
         batch_name='batch',
         min_features=600, 
         min_cells=3, 
-        n_top_features=2000, 
+        target_sum=None,
+        n_top_features=None, 
         batch_size=64, 
         chunk_size=CHUNK_SIZE,
+        processed=False,
         log=None,
     ):
     """
@@ -542,15 +565,23 @@ def load_data(
         adata.obs['batch'] = 'batch'
     adata.obs['batch'] = adata.obs['batch'].astype('category')
     
-    adata = preprocessing(
-        adata, 
-        profile=profile,
-        min_features=min_features, 
-        min_cells=min_cells, 
-        n_top_features=n_top_features,
-        chunk_size=chunk_size,
-        log=log,
-    )
+    if isinstance(n_top_features, str):
+        if os.path.isfile(n_top_features):
+            n_top_features = np.loadtxt(n_top_features, dtype=str)
+        else:
+            n_top_features = int(n_top_features)
+    
+    if not processed:
+        adata = preprocessing(
+            adata, 
+            profile=profile,
+            min_features=min_features, 
+            min_cells=min_cells, 
+            target_sum=target_sum,
+            n_top_features=n_top_features,
+            chunk_size=chunk_size,
+            log=log,
+        )
     scdata = SingleCellDataset(adata) # Wrap AnnData into Pytorch Dataset
     trainloader = DataLoader(
         scdata, 
